/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package controller;

import java.io.Serializable;
import java.util.List;
import model.Usuari;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import util.HibernateUtil;

/**
 *
 * @author Enric Arasa Curto (enric.arasa@gmail.com)
 */
public abstract class UsuariDAO{
    private  Session sesion;

    protected  void iniciaOperacion()
    {
        sesion = HibernateUtil.getSessionFactory().openSession();
        sesion.getTransaction().begin();
    }

    protected void terminaOperacion()
    {
        sesion.getTransaction().commit();
        sesion.close();
    }

    protected void manejaExcepcion(HibernateException he) throws HibernateException
    {
        sesion.getTransaction().rollback();
        throw he;
    }

    protected Session getSession()
    {
        return sesion;
    }

    public static int guardarUsuari(Object entidad) throws HibernateException
    {
        UsuariDAO dummy = new UsuariDAO(){};
        int id = -1;

        try
        {
            dummy.iniciaOperacion();
            id = (Integer)dummy.getSession().save(entidad);
            dummy.getSession().flush();
            
        }
        catch (HibernateException he)
        {
            dummy.manejaExcepcion(he);
        }
        finally
        {
            dummy.terminaOperacion();
        }
        
        return id;
    }
    
    public static void actulitzaUsuari(Object entidad) throws HibernateException
    {
        UsuariDAO dummy = new UsuariDAO(){};
        

        try
        {
            dummy.iniciaOperacion();
            dummy.getSession().update(entidad);
            dummy.getSession().flush();
        }
        catch (HibernateException he)
        {
            dummy.manejaExcepcion(he);
        }
        finally
        {
            dummy.terminaOperacion();
        }
                
    }

    public static <T> T getUsuari(Serializable id, Class<T> claseEntidad) throws HibernateException
    {
        UsuariDAO dummy = new UsuariDAO(){};

        T objetoRecuperado = null;

        try
        {
            dummy.iniciaOperacion();
            objetoRecuperado = (T) dummy.getSession().get(claseEntidad, id);
        }
        catch (HibernateException he)
        {
            dummy.manejaExcepcion(he);
        }
        finally
        {
            dummy.terminaOperacion();
        }

        return objetoRecuperado;
    }

    public static <T> List<T> getLlistaUsuaris(Class<T> claseEntidad) throws HibernateException
    {
        UsuariDAO dummy = new UsuariDAO(){};

        List<T> listaResultado = null;

        try
        {
            dummy.iniciaOperacion();
            listaResultado = dummy.getSession().createQuery("FROM " + claseEntidad.getSimpleName()).list();
        }
        catch (HibernateException he)
        {
            dummy.manejaExcepcion(he);
        }
        finally
        {
            dummy.terminaOperacion();
        }

        return listaResultado;
    }
    
    
    public static boolean existeixUsuari(String login) throws HibernateException
    {
        Usuari usuari = null;
        UsuariDAO dummy = new UsuariDAO(){};
        boolean existeix=false;

        try
        {
            dummy.iniciaOperacion();
            Query query = dummy.getSession().createQuery("FROM Usuari u WHERE u. login = :login ");
            query.setParameter("login", login);
            

            usuari = (Usuari)query.uniqueResult();
            if (usuari!=null){
                existeix=true;
            }
        }
        catch (HibernateException he)
        {
            dummy.manejaExcepcion(he);
        }
        finally
        {
            dummy.terminaOperacion();
        }

        return existeix;
    }
    
    public static boolean emailOcupat(String email) throws HibernateException
    {
        Usuari usuari = null;
        UsuariDAO dummy = new UsuariDAO(){};
        boolean existeix=false;

        try
        {
            dummy.iniciaOperacion();
            Query query = dummy.getSession().createQuery("FROM Usuari u WHERE u. email = :email ");
            query.setParameter("email", email);
            

            usuari = (Usuari)query.uniqueResult();
            if (usuari!=null){
                existeix=true;
            }
        }
        catch (HibernateException he)
        {
            dummy.manejaExcepcion(he);
        }
        finally
        {
            dummy.terminaOperacion();
        }

        return existeix;
    }
    
    public static boolean comprovarContrasenya(String login,String password) throws HibernateException
    {
        Usuari usuari = null;
        UsuariDAO dummy = new UsuariDAO(){};
        boolean correcta=false;

        try
        {
            dummy.iniciaOperacion();
            Query query = dummy.getSession().createQuery("FROM Usuari u WHERE u. login = :login AND u. password= :password");
            query.setParameter("login", login);
            query.setParameter("password", password);

            usuari = (Usuari)query.uniqueResult();
            if (usuari!=null){
                correcta=true;
            }
        }
        catch (HibernateException he)
        {
            dummy.manejaExcepcion(he);
        }
        finally
        {
            dummy.terminaOperacion();
        }

        return correcta;
    }
    
    public static Usuari getUsuari(String login,String password) throws HibernateException
    {
        Usuari usuari = null;
        UsuariDAO dummy = new UsuariDAO(){};
        

        try
        {
            dummy.iniciaOperacion();
            Query query = dummy.getSession().createQuery("FROM Usuari u WHERE u. login = :login AND u. password= :password");
            query.setParameter("login", login);
            query.setParameter("password", password);

            usuari = (Usuari)query.uniqueResult();
            
        }
        catch (HibernateException he)
        {
            dummy.manejaExcepcion(he);
        }
        finally
        {
            dummy.terminaOperacion();
        }

        return usuari;
    }
    
    public static Usuari getUsuari(String email) throws HibernateException
    {
        Usuari usuari = null;
        UsuariDAO dummy = new UsuariDAO(){};
        

        try
        {
            dummy.iniciaOperacion();
            Query query = dummy.getSession().createQuery("FROM Usuari u WHERE u. email = :email");
            query.setParameter("email", email);
            

            usuari = (Usuari)query.uniqueResult();
            
        }
        catch (HibernateException he)
        {
            dummy.manejaExcepcion(he);
        }
        finally
        {
            dummy.terminaOperacion();
        }

        return usuari;
    }
    
}