/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package controller;

import java.util.List;
import model.Client;
import model.Producte;
import model.Valoracio;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import util.HibernateUtil;
import org.hibernate.Query;

/**
 *
 * @author Enric Arasa Curto (enric.arasa@gmail.com)
 */
public abstract class ValoracioDAO{
    
    private Session sesion;
    private Transaction tx;
    
    private void iniciaOperacion() throws HibernateException{
        sesion = HibernateUtil.getSessionFactory().openSession();
        tx = sesion.beginTransaction();
    }
    
    private void manejaExcepcion(HibernateException he) throws HibernateException{
        tx.rollback();
        throw new HibernateException("Ocurrió un error en la capa de acceso a datos", he);
    }
    
    
    public static void guardaValoracio(Valoracio valoracio){ 
        
        
        ValoracioDAO dummy = new ValoracioDAO(){};

        try{ 
            dummy.iniciaOperacion(); 
            dummy.sesion.save(valoracio);
            dummy.tx.commit(); 
        }catch(HibernateException he){ 
            dummy.manejaExcepcion(he);
            throw he; 
        }finally{ 
            dummy.sesion.close(); 
        }  
        
        }
    
    public static void actualizaValoracio(Valoracio valoracio) throws HibernateException{ 
        ValoracioDAO dummy = new ValoracioDAO(){};
        try{ 
            dummy.iniciaOperacion(); 
            dummy.sesion.update(valoracio); 
            dummy.tx.commit(); 
        }catch (HibernateException he){ 
            dummy.manejaExcepcion(he); 
            throw he; 
        }finally{ 
            dummy.sesion.close(); 
        } 
    }
    
    public static void eliminaValoracio(Valoracio valoracio) throws HibernateException{ 
        ValoracioDAO dummy = new ValoracioDAO(){};
        try{ 
            dummy.iniciaOperacion(); 
            dummy.sesion.delete(valoracio); 
            dummy.tx.commit(); 
        } catch (HibernateException he){ 
            dummy.manejaExcepcion(he); 
            throw he; 
        }finally{ 
            dummy.sesion.close(); 
        } 
    }
    
    
    public static boolean comprovarSiVotat(Client client, Producte producte) throws HibernateException
    {   Valoracio valoracio=null;
        ValoracioDAO dummy = new ValoracioDAO(){};
        boolean existeix=false;

        try
        {
            dummy.iniciaOperacion();
            Query query = dummy.sesion.createQuery("FROM Valoracio v WHERE v.valoracioPK.client = :client AND "+ 
                    " v.valoracioPK.producte= :producte");
            query.setParameter("client", client);
            query.setParameter("producte", producte);
            

            valoracio = (Valoracio)query.uniqueResult();
            if (valoracio!=null){
                existeix=true;
            }
        }
        catch (HibernateException he)
        {
            dummy.manejaExcepcion(he);
        }
        finally
        {
            dummy.sesion.close();
        }

        return existeix;
    }
    
    
    public static Valoracio obtenValoracio(int idValoracio) throws HibernateException{ 
        Valoracio valoracio = null;  
        ValoracioDAO dummy = new ValoracioDAO(){};
        try{ 
            dummy.iniciaOperacion(); 
            valoracio = (Valoracio) dummy.sesion.get(Valoracio.class, idValoracio); 
        } finally{ 
            dummy.sesion.close(); 
        }  
        return valoracio; 
    }
    
    public static List<Valoracio> obtenListaValoracions() throws HibernateException{ 
        List<Valoracio> listaValoracions = null;  
        ValoracioDAO dummy = new ValoracioDAO(){};
        try { 
            dummy.iniciaOperacion(); 
            listaValoracions = dummy.sesion.createQuery("from Valoracio").list(); 
        }finally { 
            dummy.sesion.close(); 
        }  

        return listaValoracions; 
    }
    
    
}
